const fastq = require('fastq');

const { applicationData } = require('../util/db');

let matrixClient;
let targetRoomId;

const setTargetRoomId = async (roomId) => {
  targetRoomId = roomId;
  console.log('target room ID:', targetRoomId);
  await applicationData.put('targetRoomId', targetRoomId);
};

const processOutboundQueue = async ({ type, ...args }) => {
  if (!targetRoomId) return;
  switch (type) {
    case 'MatrixEvent': {
      const { eventType, content, onSend } = args;
      const eventId = await matrixClient.sendEvent(targetRoomId, eventType, content);
      onSend(targetRoomId, eventId);
      break;
    }
    case 'MatrixText': {
      const { text, onSend } = args;
      const eventId = await matrixClient.sendText(targetRoomId, text);
      onSend(targetRoomId, eventId);
      break;
    }
    default:
  }
};

const outboundQueue = fastq(processOutboundQueue, 1);
// Pause until matrixClient is set
outboundQueue.pause();

const startOutboundQueue = async (matrixClient_) => {
  matrixClient = matrixClient_;
  try {
    targetRoomId = await applicationData.get('targetRoomId');
    console.log('target room ID:', targetRoomId);
  } catch (e) {
    // No target room set
    console.warn('target room ID is not set -- will not be able to send messages until it is set. Use !target <bot-id>');
  }
  outboundQueue.resume();
};

const sendMatrixEvent = async (eventType, content) => new Promise((resolve) => {
  outboundQueue.push({
    type: 'MatrixEvent',
    eventType,
    content,
    onSend: ((roomId, eventId) => {
      resolve({ roomId, eventId });
    }),
  });
});

const sendMatrixText = async (text) => new Promise((resolve) => {
  outboundQueue.push({
    type: 'MatrixText',
    text,
    onSend: ((roomId, eventId) => {
      resolve({ roomId, eventId });
    }),
  });
});

module.exports = {
  setTargetRoomId,
  outboundQueue,
  startOutboundQueue,
  sendMatrixEvent,
  sendMatrixText,
};
